import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;

public class BiographyConverter {
	private static final String NAME_PREFIX = "NM";
	private static final String BORN_PREFIX = "DB";

	public static class Map extends
			Mapper<LongWritable, Text, LongWritable, Text> {
		
		private static final long OFFSET = 4000;
		private static final long MARGIN = 800;
		
		
		private long mapPosition(long pos) {
			return pos / OFFSET;
		}
		
		private String constructOffsetString(long offset, String line) {
			return String.valueOf(offset) + "," + line; 
		}
		
		public void map(LongWritable key, Text value, Context context)
				throws IOException, InterruptedException {
			long position = key.get();
			String str = value.toString();
			
			if(str.startsWith(NAME_PREFIX)) {
				context.write(new LongWritable(mapPosition(position)), new Text(constructOffsetString(position, str)));
			} else if(str.startsWith(BORN_PREFIX)) {
				long mappedPosition = mapPosition(position);
				if(position % OFFSET < MARGIN) {
					context.write(new LongWritable(mappedPosition - 1), new Text(constructOffsetString(position, str)));
				}
				context.write(new LongWritable(mappedPosition), new Text(constructOffsetString(position, str)));
			}
		}
	}
	
	public static class Reduce extends
			Reducer<LongWritable, Text, Text, Text> {

		private static final String UNKNOWN_BORN = "UNKNOWN";
		
		private static final String BORN_PREFIX = "B: ";
		
		public void reduce(LongWritable key, Iterable<Text> values,
				Context context) throws IOException, InterruptedException {
			int size = 0;
			List<OffsetTextPair> list = new LinkedList<OffsetTextPair>();
			for (Text val : values) {
				size++;
				list.add(new OffsetTextPair(val));
			}
			OffsetTextPair []offsetTextPairs = new OffsetTextPair[size];
			int idx = 0;
			for(OffsetTextPair val : list) {
				offsetTextPairs[idx++] = val;
			}
			Arrays.sort(offsetTextPairs);
			
			for(int i = 0; i < offsetTextPairs.length; i++) {
				if(offsetTextPairs[i].getText().startsWith(NAME_PREFIX)) {
					if(i + 1 < offsetTextPairs.length && offsetTextPairs[i + 1].getText().startsWith(BiographyConverter.BORN_PREFIX)) {
						context.write(new Text(offsetTextPairs[i].getText().substring(4)), 
								new Text(BORN_PREFIX + offsetTextPairs[i + 1].getText().substring(4)));
					} /*else {
						context.write(new Text(offsetTextPairs[i].getText().substring(4)), 
								new Text(BORN_PREFIX + UNKNOWN_BORN));
					}*/
					i++;
				}
			}
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration conf = new Configuration();

		Job job = new Job(conf, "BiographyConverter");

		job.setOutputKeyClass(LongWritable.class);
		job.setOutputValueClass(Text.class);

		job.setMapperClass(Map.class);
		job.setReducerClass(Reduce.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);
		FileInputFormat.addInputPath(job, new Path(args[0]));
		FileOutputFormat.setOutputPath(job, new Path(args[1]));
		job.setJarByClass(BiographyConverter.class);
		job.waitForCompletion(true);
	}

}
