import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Scanner;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.store.SimpleFSDirectory;
import org.apache.lucene.util.Version;

public class LuceneIndexer {

	public static final String FILES_TO_INDEX_DIRECTORY = "output//merged/";
	public static final String INDEX_DIRECTORY = "indexDirectory";

	public static final String FIELD_NAME = "name";
	public static final String FIELD_BORN = "born";
	public static final String FIELD_MOVIE = "movie";

	public static Version VERSION = Version.LUCENE_23;
	public static final int MAX_FIELD_LENGTH = 40000;
	
	private static final String BORN_PREFIX = "B: ";
	//private static final String MOVIE_PREFIX = "M: ";
	
	public static void main(String[] args) throws Exception {

		createIndex();
		searchIndex("name:\"$hort Too\"");
		searchIndex("born:(Argentina)");
		searchIndex("movie:(Homicide)");
	}

	private static void prepareIndex(IndexWriter indexWriter) throws CorruptIndexException, LockObtainFailedException, IOException {
		File dir = new File(FILES_TO_INDEX_DIRECTORY);
		File[] files = dir.listFiles();
		for (File file : files) {
			processFile(indexWriter, file);
		}
	}
	
	private static void processFile(IndexWriter indexWriter, File file) throws CorruptIndexException, IOException {
		Scanner scanner = new Scanner(new FileInputStream(file));
		String line;
		String [] splits;
		Actor lastActor = null;
		while(scanner.hasNext()) {
			line = scanner.nextLine();
			splits = line.split("\t");
			
			if(lastActor == null || !lastActor.getName().equals(splits[0])) {
				if(lastActor != null) {
					Document document = new Document();

					document.add(new Field(FIELD_NAME, lastActor.getName(), Field.Store.YES, Field.Index.ANALYZED));
					document.add(new Field(FIELD_BORN, lastActor.getBorn(), Field.Store.YES, Field.Index.ANALYZED));
					for(String movie: lastActor.getMovies()) {
						document.add(new Field(FIELD_MOVIE, movie, Field.Store.YES, Field.Index.ANALYZED));
					}

					indexWriter.addDocument(document);
				}
				lastActor = new Actor(splits[0]);
			}

			if(splits[1].startsWith(BORN_PREFIX)) {
				lastActor.setBorn(splits[1].substring(3));
			} else {
				lastActor.addMovie(splits[1].substring(3));
			}
		}
	}
	
	public static void createIndex() throws CorruptIndexException, LockObtainFailedException, IOException {
		Analyzer analyzer = new StandardAnalyzer(VERSION);
		boolean recreateIndexIfExists = true;
		Directory indexDirectory = new SimpleFSDirectory(new File(INDEX_DIRECTORY));
		
		IndexWriter indexWriter = new IndexWriter(indexDirectory, analyzer, recreateIndexIfExists, new IndexWriter.MaxFieldLength(MAX_FIELD_LENGTH));
		prepareIndex(indexWriter);
		indexWriter.optimize();
		indexWriter.close();
	}

	private static final int NUMBER_OF_DOCS = 100;
	public static void searchIndex(String searchString) throws IOException, ParseException {
		System.out.println("Searching for '" + searchString + "'");
		Directory directory = FSDirectory.open(new File(INDEX_DIRECTORY));
		IndexReader indexReader = IndexReader.open(directory);
		IndexSearcher indexSearcher = new IndexSearcher(indexReader);
		Analyzer analyzer = new StandardAnalyzer(VERSION);
		QueryParser queryParser = new QueryParser(VERSION, FIELD_MOVIE, analyzer);
		
		Query query = queryParser.parse(searchString);
		ScoreDoc []scoreDocs = indexSearcher.search(query, NUMBER_OF_DOCS).scoreDocs;

		for(int i = 0 ; i < scoreDocs.length; i++){
			Document document = indexSearcher.doc(scoreDocs[i].doc);
			System.out.println("Hit: ");
			System.out.println("\tName: " + document.get(FIELD_NAME));
			System.out.println("\tBorn: " + document.get(FIELD_BORN));
			for(Fieldable movie: document.getFieldables(FIELD_MOVIE))	{
				System.out.println("\tMovie: " + movie.stringValue());
			}
		}

	}

}
